from abc import (
    ABCMeta,
    abstractmethod
)


class Subscriber(metaclass=ABCMeta):

    @abstractmethod
    def handle(self, event):
        pass

    @abstractmethod
    def _events_subscribed_to(self):
        pass

    def is_subscribed_to(self, event):
        for subscribed_event in self._events_subscribed_to():
            if isinstance(event, subscribed_event):
                return True

        return False



from uuid import uuid4


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class Publisher(metaclass=Singleton):

    def __init__(self):
        self.__subscribers = {}

    @property
    def subscribers(self):
        return self.__subscribers.copy()

    def subscribe(self, subscriber, subscriber_uuid=None):
        uuid = subscriber_uuid or str(uuid4())

        if uuid in list(self.__subscribers.keys()):
            raise Exception("There already exists a subscriber with this uuid ({})".format(uuid))

        self.__subscribers[uuid] = subscriber

        return uuid

    def publish(self, event):
        for uuid, a_subscriber in self.__subscribers.items():
            if a_subscriber.is_subscribed_to(event):
                a_subscriber.handle(event)

    def clear(self):
        self.__subscribers.clear()



from setuptools import setup


setup(
    name="domain-events",
    version="2.3.0",
    license="GPLv3",
    author="Isagri S.L.U.",
    author_email="devs.es@groupeisagri.com",
    description="A lightweight library with an implementation of Pub-Sub.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/agroptima/domain-events",
    download_url="https://github.com/agroptima/domain-events/releases",
    keywords=["python", "ddd"],
    packages=["domain_events"],
    install_requires=["pytz"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Utilities",
    ],
)



import datetime

import pytz


class Event(object):

    def __init__(self, *args, **kwargs):
        self.occurred_on = kwargs.get('occurred_on', datetime.datetime.now(tz=pytz.utc))
        self.type_name = self.type_name()

    @classmethod
    def type_name(cls):
        return "{}.{}".format(cls.__module__, cls.__name__)
